@@ -178,7 +178,7 @@ class ResourceUtilsSuite extends SparkFunSuite
   test("list resource ids") {
     val conf = new SparkConf
     conf.set(DRIVER_GPU_ID.amountConf, "2")
-    var resources = listResourceIds(conf, SPARK_DRIVER_PREFIX)
+    val resources = listResourceIds(conf, SPARK_DRIVER_PREFIX)
     assert(resources.size === 1, "should only have GPU for resource")
     assert(resources(0).resourceName == GPU, "name should be gpu")
 
