@@ -454,7 +454,7 @@ if [[ "$1" == "publish-release" ]]; then
   if ! is_dry_run; then
     echo "Creating Nexus staging repository"
     repo_request="<promoteRequest><data><description>Apache Spark $SPARK_VERSION (commit $git_hash)</description></data></promoteRequest>"
-    out=$(curl -X POST -d "$repo_request" -u $ASF_USERNAME:$ASF_PASSWORD \
+    out=$(curl --retry 10 --retry-connrefused --fail -X POST -d "$repo_request" -u $ASF_USERNAME:$ASF_PASSWORD \
       -H "Content-Type:application/xml" -v \
       $NEXUS_ROOT/profiles/$NEXUS_PROFILE/start)
     staged_repo_id=$(echo $out | sed -e "s/.*\(orgapachespark-[0-9]\{4\}\).*/\1/")
@@ -499,18 +499,38 @@ if [[ "$1" == "publish-release" ]]; then
   if ! is_dry_run; then
     nexus_upload=$NEXUS_ROOT/deployByRepositoryId/$staged_repo_id
     echo "Uploading files to $nexus_upload"
-    for file in $(find . -type f)
-    do
-      # strip leading ./
-      file_short=$(echo $file | sed -e "s/\.\///")
-      dest_url="$nexus_upload/org/apache/spark/$file_short"
-      echo "  Uploading $file_short"
-      curl -u $ASF_USERNAME:$ASF_PASSWORD --upload-file $file_short $dest_url
-    done
+
+    # Temp file to track errors
+    error_flag_file=$(mktemp)
+
+    find . -type f | sed -e 's|^\./||' | \
+    xargs -P 4 -n 1 -I {} bash -c '
+      file_short="{}"
+      dest_url="'$NEXUS_ROOT'/deployByRepositoryId/'$staged_repo_id'/org/apache/spark/$file_short"
+      echo "[START] $file_short"
+
+      if curl --retry 10 --retry-connrefused --fail -u "$ASF_USERNAME:$ASF_PASSWORD" \
+          --upload-file "$file_short" "$dest_url"; then
+        echo "[ OK  ] $file_short"
+      else
+        echo "[FAIL ] $file_short"
+        echo "fail" >> '"$error_flag_file"'
+      fi
+    '
+
+    # Check if any failures were recorded
+    if [ -s "$error_flag_file" ]; then
+      echo "One or more uploads failed."
+      rm "$error_flag_file"
+      exit 1
+    else
+      echo "All uploads succeeded."
+      rm "$error_flag_file"
+    fi
 
     echo "Closing nexus staging repository"
     repo_request="<promoteRequest><data><stagedRepositoryId>$staged_repo_id</stagedRepositoryId><description>Apache Spark $SPARK_VERSION (commit $git_hash)</description></data></promoteRequest>"
-    out=$(curl -X POST -d "$repo_request" -u $ASF_USERNAME:$ASF_PASSWORD \
+    out=$(curl --retry 10 --retry-connrefused --fail -X POST -d "$repo_request" -u $ASF_USERNAME:$ASF_PASSWORD \
       -H "Content-Type:application/xml" -v \
       $NEXUS_ROOT/profiles/$NEXUS_PROFILE/finish)
     echo "Closed Nexus staging repository: $staged_repo_id"
