@@ -136,6 +136,7 @@ if not (ENV['SKIP_API'] == '1')
     mkdir_p "api/python"
 
     puts "cp -r ../python/docs/build/html/. api/python"
+    rm_r("../python/docs/build/html/_sources")
     cp_r("../python/docs/build/html/.", "api/python")
   end
 
@@ -152,11 +153,8 @@ if not (ENV['SKIP_API'] == '1')
     puts "Making directory api/R"
     mkdir_p "api/R"
 
-    puts "cp -r ../R/pkg/html/. api/R"
-    cp_r("../R/pkg/html/.", "api/R")
-
-    puts "cp ../R/pkg/DESCRIPTION api"
-    cp("../R/pkg/DESCRIPTION", "api")
+    puts "cp -r ../R/pkg/docs/. api/R"
+    cp_r("../R/pkg/docs/.", "api/R")
   end
 
   if not (ENV['SKIP_SQLDOC'] == '1')
