@@ -19,21 +19,24 @@ package org.apache.spark.repl
 
 import java.io._
 
-import org.scalatest.BeforeAndAfterAll
+import scala.tools.nsc.interpreter.SimpleReader
 
 import org.apache.spark.{SparkContext, SparkFunSuite}
 
-class Repl2Suite extends SparkFunSuite with BeforeAndAfterAll {
+class Repl2Suite extends SparkFunSuite {
   test("propagation of local properties") {
     // A mock ILoop that doesn't install the SIGINT handler.
-    class ILoop(out: PrintWriter) extends SparkILoop(null, out)
+    class ILoop(out: PrintWriter) extends SparkILoop(None, out) {
+      settings = new scala.tools.nsc.Settings
+      settings.usejavacp.value = true
+      org.apache.spark.repl.Main.interp = this
+      in = SimpleReader()
+    }
 
     val out = new StringWriter()
     Main.interp = new ILoop(new PrintWriter(out))
     Main.sparkContext = new SparkContext("local", "repl-test")
-    val settings = new scala.tools.nsc.Settings
-    settings.usejavacp.value = true
-    Main.interp.createInterpreter(settings)
+    Main.interp.createInterpreter()
 
     Main.sparkContext.setLocalProperty("someKey", "someValue")
 
