@@ -21,10 +21,10 @@ import java.math.{BigDecimal, BigInteger}
 import java.nio.ByteBuffer
 
 import scala.annotation.tailrec
-import scala.reflect.runtime.universe.TypeTag
 
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions._
+import org.apache.spark.sql.catalyst.types.{PhysicalArrayType, PhysicalBinaryType, PhysicalBooleanType, PhysicalByteType, PhysicalCalendarIntervalType, PhysicalDataType, PhysicalDecimalType, PhysicalDoubleType, PhysicalFloatType, PhysicalIntegerType, PhysicalLongType, PhysicalMapType, PhysicalNullType, PhysicalShortType, PhysicalStringType, PhysicalStructType}
 import org.apache.spark.sql.errors.QueryExecutionErrors
 import org.apache.spark.sql.types._
 import org.apache.spark.unsafe.Platform
@@ -110,8 +110,8 @@ private[columnar] object ByteBufferHelper {
  */
 private[columnar] sealed abstract class ColumnType[JvmType] {
 
-  // The catalyst data type of this column.
-  def dataType: DataType
+  // The catalyst physical data type of this column.
+  def dataType: PhysicalDataType
 
   // Default size in bytes for one element of type T (e.g. 4 for `Int`).
   def defaultSize: Int
@@ -179,7 +179,7 @@ private[columnar] sealed abstract class ColumnType[JvmType] {
 
 private[columnar] object NULL extends ColumnType[Any] {
 
-  override def dataType: DataType = NullType
+  override def dataType: PhysicalDataType = PhysicalNullType
   override def defaultSize: Int = 0
   override def append(v: Any, buffer: ByteBuffer): Unit = {}
   override def extract(buffer: ByteBuffer): Any = null
@@ -187,18 +187,13 @@ private[columnar] object NULL extends ColumnType[Any] {
   override def getField(row: InternalRow, ordinal: Int): Any = null
 }
 
-private[columnar] abstract class NativeColumnType[T <: AtomicType](
+private[columnar] abstract class NativeColumnType[T <: PhysicalDataType](
     val dataType: T,
     val defaultSize: Int)
   extends ColumnType[T#InternalType] {
-
-  /**
-   * Scala TypeTag. Can be used to create primitive arrays and hash tables.
-   */
-  def scalaTag: TypeTag[dataType.InternalType] = dataType.tag
 }
 
-private[columnar] object INT extends NativeColumnType(IntegerType, 4) {
+private[columnar] object INT extends NativeColumnType(PhysicalIntegerType, 4) {
   override def append(v: Int, buffer: ByteBuffer): Unit = {
     buffer.putInt(v)
   }
@@ -228,7 +223,7 @@ private[columnar] object INT extends NativeColumnType(IntegerType, 4) {
   }
 }
 
-private[columnar] object LONG extends NativeColumnType(LongType, 8) {
+private[columnar] object LONG extends NativeColumnType(PhysicalLongType, 8) {
   override def append(v: Long, buffer: ByteBuffer): Unit = {
     buffer.putLong(v)
   }
@@ -257,7 +252,7 @@ private[columnar] object LONG extends NativeColumnType(LongType, 8) {
   }
 }
 
-private[columnar] object YEAR_MONTH_INTERVAL extends NativeColumnType(YearMonthIntervalType(), 4) {
+private[columnar] object YEAR_MONTH_INTERVAL extends NativeColumnType(PhysicalIntegerType, 4) {
   override def append(v: Int, buffer: ByteBuffer): Unit = {
     buffer.putInt(v)
   }
@@ -290,7 +285,7 @@ private[columnar] object YEAR_MONTH_INTERVAL extends NativeColumnType(YearMonthI
   }
 }
 
-private[columnar] object DAY_TIME_INTERVAL extends NativeColumnType(DayTimeIntervalType(), 8) {
+private[columnar] object DAY_TIME_INTERVAL extends NativeColumnType(PhysicalLongType, 8) {
   override def append(v: Long, buffer: ByteBuffer): Unit = {
     buffer.putLong(v)
   }
@@ -322,7 +317,7 @@ private[columnar] object DAY_TIME_INTERVAL extends NativeColumnType(DayTimeInter
   }
 }
 
-private[columnar] object FLOAT extends NativeColumnType(FloatType, 4) {
+private[columnar] object FLOAT extends NativeColumnType(PhysicalFloatType, 4) {
   override def append(v: Float, buffer: ByteBuffer): Unit = {
     buffer.putFloat(v)
   }
@@ -351,7 +346,7 @@ private[columnar] object FLOAT extends NativeColumnType(FloatType, 4) {
   }
 }
 
-private[columnar] object DOUBLE extends NativeColumnType(DoubleType, 8) {
+private[columnar] object DOUBLE extends NativeColumnType(PhysicalDoubleType, 8) {
   override def append(v: Double, buffer: ByteBuffer): Unit = {
     buffer.putDouble(v)
   }
@@ -380,7 +375,7 @@ private[columnar] object DOUBLE extends NativeColumnType(DoubleType, 8) {
   }
 }
 
-private[columnar] object BOOLEAN extends NativeColumnType(BooleanType, 1) {
+private[columnar] object BOOLEAN extends NativeColumnType(PhysicalBooleanType, 1) {
   override def append(v: Boolean, buffer: ByteBuffer): Unit = {
     buffer.put(if (v) 1: Byte else 0: Byte)
   }
@@ -407,7 +402,7 @@ private[columnar] object BOOLEAN extends NativeColumnType(BooleanType, 1) {
   }
 }
 
-private[columnar] object BYTE extends NativeColumnType(ByteType, 1) {
+private[columnar] object BYTE extends NativeColumnType(PhysicalByteType, 1) {
   override def append(v: Byte, buffer: ByteBuffer): Unit = {
     buffer.put(v)
   }
@@ -436,7 +431,7 @@ private[columnar] object BYTE extends NativeColumnType(ByteType, 1) {
   }
 }
 
-private[columnar] object SHORT extends NativeColumnType(ShortType, 2) {
+private[columnar] object SHORT extends NativeColumnType(PhysicalShortType, 2) {
   override def append(v: Short, buffer: ByteBuffer): Unit = {
     buffer.putShort(v)
   }
@@ -497,7 +492,7 @@ private[columnar] trait DirectCopyColumnType[JvmType] extends ColumnType[JvmType
 }
 
 private[columnar] object STRING
-  extends NativeColumnType(StringType, 8) with DirectCopyColumnType[UTF8String] {
+  extends NativeColumnType(PhysicalStringType, 8) with DirectCopyColumnType[UTF8String] {
 
   override def actualSize(row: InternalRow, ordinal: Int): Int = {
     row.getUTF8String(ordinal).numBytes() + 4
@@ -537,7 +532,7 @@ private[columnar] object STRING
 }
 
 private[columnar] case class COMPACT_DECIMAL(precision: Int, scale: Int)
-  extends NativeColumnType(DecimalType(precision, scale), 8) {
+  extends NativeColumnType(PhysicalDecimalType(precision, scale), 8) {
 
   override def extract(buffer: ByteBuffer): Decimal = {
     Decimal(ByteBufferHelper.getLong(buffer), precision, scale)
@@ -606,7 +601,7 @@ private[columnar] sealed abstract class ByteArrayColumnType[JvmType](val default
 
 private[columnar] object BINARY extends ByteArrayColumnType[Array[Byte]](16) {
 
-  def dataType: DataType = BinaryType
+  def dataType: PhysicalDataType = PhysicalBinaryType
 
   override def setField(row: InternalRow, ordinal: Int, value: Array[Byte]): Unit = {
     row.update(ordinal, value)
@@ -627,7 +622,7 @@ private[columnar] object BINARY extends ByteArrayColumnType[Array[Byte]](16) {
 private[columnar] case class LARGE_DECIMAL(precision: Int, scale: Int)
   extends ByteArrayColumnType[Decimal](12) {
 
-  override val dataType: DataType = DecimalType(precision, scale)
+  override val dataType: PhysicalDataType = PhysicalDecimalType(precision, scale)
 
   override def getField(row: InternalRow, ordinal: Int): Decimal = {
     row.getDecimal(ordinal, precision, scale)
@@ -657,7 +652,7 @@ private[columnar] object LARGE_DECIMAL {
   }
 }
 
-private[columnar] case class STRUCT(dataType: StructType)
+private[columnar] case class STRUCT(dataType: PhysicalStructType)
   extends ColumnType[UnsafeRow] with DirectCopyColumnType[UnsafeRow] {
 
   private val numOfFields: Int = dataType.fields.length
@@ -697,7 +692,7 @@ private[columnar] case class STRUCT(dataType: StructType)
   override def clone(v: UnsafeRow): UnsafeRow = v.copy()
 }
 
-private[columnar] case class ARRAY(dataType: ArrayType)
+private[columnar] case class ARRAY(dataType: PhysicalArrayType)
   extends ColumnType[UnsafeArrayData] with DirectCopyColumnType[UnsafeArrayData] {
 
   override def defaultSize: Int = 28
@@ -736,7 +731,7 @@ private[columnar] case class ARRAY(dataType: ArrayType)
   override def clone(v: UnsafeArrayData): UnsafeArrayData = v.copy()
 }
 
-private[columnar] case class MAP(dataType: MapType)
+private[columnar] case class MAP(dataType: PhysicalMapType)
   extends ColumnType[UnsafeMapData] with DirectCopyColumnType[UnsafeMapData] {
 
   override def defaultSize: Int = 68
@@ -776,7 +771,7 @@ private[columnar] case class MAP(dataType: MapType)
 
 private[columnar] object CALENDAR_INTERVAL extends ColumnType[CalendarInterval] {
 
-  override def dataType: DataType = CalendarIntervalType
+  override def dataType: PhysicalDataType = PhysicalCalendarIntervalType
 
   override def defaultSize: Int = 16
 
@@ -830,9 +825,9 @@ private[columnar] object ColumnType {
       case i: CalendarIntervalType => CALENDAR_INTERVAL
       case dt: DecimalType if dt.precision <= Decimal.MAX_LONG_DIGITS => COMPACT_DECIMAL(dt)
       case dt: DecimalType => LARGE_DECIMAL(dt)
-      case arr: ArrayType => ARRAY(arr)
-      case map: MapType => MAP(map)
-      case struct: StructType => STRUCT(struct)
+      case arr: ArrayType => ARRAY(PhysicalArrayType(arr.elementType, arr.containsNull))
+      case map: MapType => MAP(PhysicalMapType(map.keyType, map.valueType, map.valueContainsNull))
+      case struct: StructType => STRUCT(PhysicalStructType(struct.fields))
       case udt: UserDefinedType[_] => ColumnType(udt.sqlType)
       case other => throw QueryExecutionErrors.unsupportedTypeError(other)
     }
